# -*- coding: utf-8 -*-
"""
Created on Mon Nov 11 11:58:44 2024

@author: Giovanni
"""

import numpy as np
import matplotlib.pyplot as plt
from PCA_reconstruction import PCA_reconstruction  # Import the function

# Load validation data
true_signal = np.loadtxt('validation.dat')
N, n = true_signal.shape

# Define variance levels to test
variance_levels = [0.9, 0.95, 0.98, 0.99, 0.995, 1.0]

# Initialize lists to store results
rmse = []
rmse_sim = []
robustness = []
number_PC = []

# Turn off interactive plotting
# plt.ioff()

# Perform PCA reconstruction for each variance level
for v in variance_levels:
    # Perform PCA reconstruction on 'validation.dat'
    test_reconstruction_n, n_PC_n, rmse_n = PCA_reconstruction('train.dat', 'validation.dat', v)
    rmse.append(rmse_n)
    
    # Perform PCA reconstruction on 'validation_sim.dat'
    # test_reconstruction, n_PC, rmse_a = PCA_reconstruction('train.dat', 'val_anomaly.dat', v)
    test_reconstruction, n_PC, rmse_a = PCA_reconstruction('train.dat', 'validation_sim.dat', v)
    rmse_sim.append(rmse_sim)
    number_PC.append(n_PC)

    # Calculate robustness metric
    robustness_val = np.sqrt(np.sum((test_reconstruction[:, 5] - true_signal[:, 5]) ** 2)) / N
    # robustness_val = np.sqrt(np.sum((test_reconstruction[:, 0] - true_signal[:, 0]) ** 2)) / N
    robustness.append(robustness_val)

    # Close any generated figures
    plt.close('all')

# Turn on interactive plotting
plt.ion()

# Plot reconstruction error and robustness against the number of principal components
plt.figure()
plt.plot(number_PC, rmse, marker='o', markerfacecolor='blue', label='Reconstruction Error')
plt.xlabel('Number of Principal Components')
plt.plot(number_PC, robustness, marker='o', markerfacecolor='red', label='Robustness')
plt.legend()
plt.show()
