# -*- coding: utf-8 -*-
"""
Created on Mon Nov 11 12:48:15 2024

@author: Giovanni
"""

import numpy as np
import matplotlib.pyplot as plt
from sympy import symbols, log

def SPRT(test_data_name, test_data_rec_name, alpha, beta, mean_H1, sigma):
    # Define symbolic variable for SPRT boundaries
    x = symbols('x')

    # Load true and reconstructed data
    true = np.loadtxt(test_data_name)
    reconstructed = np.loadtxt(test_data_rec_name)

    # SPRT parameter settings
    n_patterns_val = true.shape[0]
    alfa = alpha
    beta = beta
    meanH_zero = 0  # Mean in normal conditions
    std_residuo = sigma  # Standard deviation in faulty condition
    mu_fault = mean_H1

    # Boundary limit calculations
    Limit_UP = log((1 - beta) / alfa)  # Upper limit
    Limit_CENTER_UP = log(beta / (1 - alfa))  # Lower limit

    # SPRT detection - Positive drift
    meanH_one = mu_fault
    sum_up = np.zeros(n_patterns_val)
    SPRT_Index = np.zeros(n_patterns_val)
    condition_up = np.zeros(n_patterns_val)

    for n_patt in range(n_patterns_val):
        yk = true[n_patt] - reconstructed[n_patt]
        if n_patt == 0:
            sum_up[n_patt] = yk - (meanH_one / 2)
        else:
            sum_up[n_patt] = sum_up[n_patt - 1] + (yk - (meanH_one / 2))
        
        # SPRT index calculation
        SPRT_Index[n_patt] = (meanH_one / (std_residuo ** 2)) * sum_up[n_patt]

        # Decision making based on thresholds
        if SPRT_Index[n_patt] < Limit_UP and SPRT_Index[n_patt] > Limit_CENTER_UP:
            condition_up[n_patt] = 0  # Indecision
        elif SPRT_Index[n_patt] < Limit_CENTER_UP:
            condition_up[n_patt] = 1  # Normal condition
            SPRT_Index[n_patt] = Limit_CENTER_UP
            sum_up[n_patt] = 0
        else:
            condition_up[n_patt] = 2  # Fault detected
            SPRT_Index[n_patt] = Limit_UP
            sum_up[n_patt] = 0

    # Fault and normal distribution analysis for mean time calculation
    fault_indices = np.where(condition_up == 2)[0]
    normal_indices = np.where(condition_up == 1)[0]

    if fault_indices.size == 0:
        if normal_indices.size == 0:
            dist = n_patterns_val
        else:
            dist = np.diff(normal_indices)[::-1]
    else:
        if fault_indices.size > normal_indices.size:
            dist = np.diff(fault_indices)[::-1]
        else:
            dist = np.diff(normal_indices)[::-1]
    
    mean_time = np.mean(dist) if dist.size > 0 else 0
    Empirical_ASN = mean_time

    # Plotting results
    plt.figure()
    plt.subplot(2, 1, 1)
    plt.plot(true, linewidth=2, label='True Signal Value')
    plt.plot(reconstructed, 'rx', linewidth=2, label='Reconstruction')
    plt.grid()
    plt.legend(loc='best')
    plt.xlabel('Time')
    plt.ylabel('Signal Value')

    plt.subplot(2, 1, 2)
    plt.plot(true - reconstructed, label='Residuals')
    plt.grid()
    plt.xlabel('Time')
    plt.ylabel('Residuals')
    plt.show()

    plt.figure()
    plt.axhline(y=Limit_UP, color='r', linestyle='--', linewidth=3, label='Upper Threshold')
    plt.axhline(y=Limit_CENTER_UP, color='b', linestyle='--', linewidth=3, label='Lower Threshold')
    plt.plot(SPRT_Index, 'k.', markersize=15, label='SPRT Value')
    plt.grid()
    plt.legend(loc='best')
    plt.xlabel('Time')
    plt.ylabel('SPRT Index')
    plt.show()

    # Fault statistics
    condition_faults_after_40 = np.sum(condition_up[40:] == 2)
    condition_u = np.sum(condition_up == 2)
    condition_d = np.sum(condition_up == 1)
    Q = np.mean(condition_d)
    P = np.mean(condition_u)
    return SPRT_Index
